'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const AWS = require('aws-sdk');
const fs = require('fs');
const path = require('path');
const tar = require('tar');
const os = require('os');

class ChromeInstaller {
  constructor({
    accessKeyId,
    secretAccessKey,
    s3Bucket,
    s3Key,
    executePath,
    debug
  }) {
    // url to download chrome
    this.setupChromePath = os.tmpdir();
    this.executablePath = path.join(this.setupChromePath, executePath);
    AWS.config.update({ accessKeyId, secretAccessKey });
    this.s3 = new AWS.S3({ apiVersion: '2006-03-01' });
    this.s3Bucket = s3Bucket;
    this.s3Key = s3Key;
    if (typeof debug !== 'boolean') {
      this.debug = true;
    } else {
      this.debug = debug;
    }
  }
  log(message, value) {
    if (this.debug) {
      if (value) {
        console.log(message, value);
      } else {
        console.log(message);
      }
    }
  }
  setupChrome() {
    var _this = this;

    return _asyncToGenerator(function* () {
      const alreadyInstalled = yield _this.existsExecutableChrome();
      if (!alreadyInstalled) {
        _this.log('Chrome not yet installed');
        try {
          const res = yield _this.setupFromS3();
          _this.log('Download s3', res);
          _this.log('File download exists', alreadyInstalled);
          return true;
        } catch (e) {
          _this.log('An error occured when downloading Chrome from S3');
          _this.log(e);
          return true;
        }
      } else {
        _this.log('Chrome already downloaded');
        return true;
      }
    })();
  }

  existsExecutableChrome() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      return new Promise(function (resolve) {
        fs.access(_this2.executablePath, fs.constants.F_OK, function (err) {
          if (!err) {
            resolve(true);
          } else {
            resolve(false);
          }
        });
      });
    })();
  }

  setupFromS3() {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      return new Promise(function (resolve, reject) {
        const params = {
          Bucket: _this3.s3Bucket,
          Key: _this3.s3Key
        };
        _this3.log('Started download at', new Date().toString());
        _this3.s3.getObject(params).createReadStream().on('error', function (err) {
          return reject(err);
        }).pipe(tar.x({
          C: _this3.setupChromePath
        })).on('error', function (err) {
          return reject(err);
        }).on('finish', function () {
          _this3.log('Finished at ', new Date().toString());
        }).on('end', function () {
          _this3.log('Done at ', new Date().toString());
          resolve('done');
        });
      });
    })();
  }
}

module.exports = ChromeInstaller;